@@ -18,9 +18,9 @@ module Happstack.Server.Internal.Types
      readDec', fromReadS, readM, FromReqURI(..)
     ) where
 
-
 import Control.Monad.Error (Error(strMsg))
 import Control.Monad.Trans (MonadIO(liftIO))
+import qualified Control.Concurrent.Thread.Group as TG
 import Control.Concurrent.MVar
 import qualified Data.Map as M
 import Data.Data (Data)
@@ -68,11 +68,20 @@ isHTTP1_0 rq =
       _               -> False
 
 -- | Should the connection be used for further messages after this.
--- | isHTTP1_0 && hasKeepAlive || isHTTP1_1 && hasNotConnectionClose
+-- isHTTP1_0 && hasKeepAlive || isHTTP1_1 && hasNotConnectionClose
+--
+-- In addition to this rule All 1xx (informational), 204 (no content),
+-- and 304 (not modified) responses MUST NOT include a message-body
+-- and therefore are eligible for connection keep-alive.
 continueHTTP :: Request -> Response -> Bool
-continueHTTP rq res =
-    (isHTTP1_0 rq && checkHeaderBS connectionC keepaliveC rq   && rsfLength (rsFlags res) == ContentLength) ||
-    (isHTTP1_1 rq && not (checkHeaderBS connectionC closeC rq) && rsfLength (rsFlags res) /= NoContentLength)
+continueHTTP rq rs =
+    (isHTTP1_0 rq && checkHeaderBS connectionC keepaliveC rq   &&
+       (rsfLength (rsFlags rs) == ContentLength || isNoMessageBodyResponse rs)) ||
+    (isHTTP1_1 rq && not (checkHeaderBS connectionC closeC rq) &&
+       (rsfLength (rsFlags rs) /= NoContentLength || isNoMessageBodyResponse rs))
+  where
+    isNoMessageBodyCode code = (code >= 100 && code <= 199) || code == 204 || code == 304
+    isNoMessageBodyResponse rs = isNoMessageBodyCode (rsCode rs) && L.null (rsBody rs)
 
 -- | function to log access requests (see also: 'logMAccess')
 -- type LogAccess time =
@@ -84,7 +93,7 @@ continueHTTP rq res =
 --     -> Integer -- ^ size
 --     -> String  -- ^ referer
 --     -> String  -- ^ userAgent
---     -> IO ()) 
+--     -> IO ())
 type LogAccess time =
     (   String
      -> String
@@ -94,23 +103,25 @@ type LogAccess time =
      -> Integer
      -> String
      -> String
-     -> IO ()) 
+     -> IO ())
 
 -- | HTTP configuration
 data Conf = Conf
-    { port       :: Int             -- ^ Port for the server to listen on.
-    , validator  :: Maybe (Response -> IO Response) -- ^ a function to validate the output on-the-fly
-    , logAccess  :: forall t. FormatTime t => Maybe (LogAccess t) -- ^ function to log access requests (see also: 'logMAccess')
-    , timeout    :: Int             -- ^ number of seconds to wait before killing an inactive thread
+    { port        :: Int             -- ^ Port for the server to listen on.
+    , validator   :: Maybe (Response -> IO Response) -- ^ a function to validate the output on-the-fly
+    , logAccess   :: forall t. FormatTime t => Maybe (LogAccess t) -- ^ function to log access requests (see also: 'logMAccess')
+    , timeout     :: Int             -- ^ number of seconds to wait before killing an inactive thread
+    , threadGroup :: Maybe TG.ThreadGroup -- ^ ThreadGroup for registering spawned threads for handling requests
     }
 
 -- | Default configuration contains no validator and the port is set to 8000
 nullConf :: Conf
 nullConf =
-    Conf { port      = 8000
-         , validator = Nothing
-         , logAccess = Just logMAccess
-         , timeout   = 30
+    Conf { port        = 8000
+         , validator   = Nothing
+         , logAccess   = Just logMAccess
+         , timeout     = 30
+         , threadGroup = Nothing
          }
 
 -- | log access requests using hslogger and apache-style log formatting
@@ -487,5 +498,5 @@ instance FromReqURI Bool    where
       "0"     -> Just False
       "false" -> Just False
       "1"     -> Just True
-      "True"  -> Just True
+      "true"  -> Just True
       _       -> Nothing
